@@ -38,9 +38,9 @@
 #include "relationfetch.h"
 
 #include <private/scope_p.h>
+#include <private/dbus_p.h>
 
 #include <shared/akdebug.h>
-#include <shared/akdbus.h>
 
 #include <QtCore/QLocale>
 #include <QtCore/QStringList>
@@ -51,9 +51,27 @@
 #include <QtSql/QSqlError>
 #include <QtSql/QSqlQuery>
 
+#include <QtCore/QElapsedTimer>
+
 using namespace Akonadi;
 using namespace Akonadi::Server;
 
+#define ENABLE_FETCH_PROFILING 0
+#if ENABLE_FETCH_PROFILING
+    #define BEGIN_TIMER(name) \
+        QElapsedTimer name##Timer; \
+        name##Timer.start();
+
+    #define END_TIMER(name) \
+        const double name##Elapsed = name##Timer.nsecsElapsed() / 1000000.0;
+    #define PROF_INC(name) \
+        ++name;
+#else
+    #define BEGIN_TIMER(name)
+    #define END_TIMER(name)
+    #define PROF_INC(name)
+#endif
+
 FetchHelper::FetchHelper(Connection *connection, const Scope &scope,
                          const Protocol::FetchScope &fetchScope)
     : mConnection(connection)
@@ -65,8 +83,7 @@ FetchHelper::FetchHelper(Connection *connection, const Scope &scope,
 
 enum PartQueryColumns {
     PartQueryPimIdColumn,
-    PartQueryTypeNamespaceColumn,
-    PartQueryTypeNameColumn,
+    PartQueryTypeIdColumn,
     PartQueryDataColumn,
     PartQueryExternalColumn,
     PartQueryVersionColumn,
@@ -80,10 +97,8 @@ QSqlQuery FetchHelper::buildPartQuery(const QVector<QByteArray> &partList, bool
 
     if (!partList.isEmpty() || allPayload || allAttrs) {
         partQuery.addJoin(QueryBuilder::InnerJoin, Part::tableName(), PimItem::idFullColumnName(), Part::pimItemIdFullColumnName());
-        partQuery.addJoin(QueryBuilder::InnerJoin, PartType::tableName(), Part::partTypeIdFullColumnName(), PartType::idFullColumnName());
         partQuery.addColumn(PimItem::idFullColumnName());
-        partQuery.addColumn(PartType::nsFullColumnName());
-        partQuery.addColumn(PartType::nameFullColumnName());
+        partQuery.addColumn(Part::partTypeIdFullColumnName());
         partQuery.addColumn(Part::dataFullColumnName());
         partQuery.addColumn(Part::externalFullColumnName());
         partQuery.addColumn(Part::versionFullColumnName());
@@ -91,27 +106,23 @@ QSqlQuery FetchHelper::buildPartQuery(const QVector<QByteArray> &partList, bool
 
         partQuery.addSortColumn(PimItem::idFullColumnName(), Query::Descending);
 
-        Query::Condition cond(Query::Or);
-        if (!partList.isEmpty()) {
-            QStringList partNameList;
+        if (!partList.isEmpty() || allPayload || allAttrs) {
+            Query::Condition cond(Query::Or);
             Q_FOREACH (const QByteArray &b, partList) {
                 if (b.startsWith("PLD") || b.startsWith("ATR")) {
-                    partNameList.push_back(QString::fromLatin1(b));
+                    cond.addValueCondition(Part::partTypeIdFullColumnName(), Query::Equals, PartTypeHelper::fromFqName(b).id());
                 }
             }
-            if (!partNameList.isEmpty()) {
-                cond.addCondition(PartTypeHelper::conditionFromFqNames(partNameList));
+            if (allPayload || allAttrs) {
+                partQuery.addJoin(QueryBuilder::InnerJoin, PartType::tableName(), Part::partTypeIdFullColumnName(), PartType::idFullColumnName());
+                if (allPayload) {
+                    cond.addValueCondition(PartType::nsFullColumnName(), Query::Equals, QStringLiteral("PLD"));
+                }
+                if (allAttrs) {
+                    cond.addValueCondition(PartType::nsFullColumnName(), Query::Equals, QStringLiteral("ATR"));
+                }
             }
-        }
 
-        if (allPayload) {
-            cond.addValueCondition(PartType::nsFullColumnName(), Query::Equals, QLatin1String("PLD"));
-        }
-        if (allAttrs) {
-            cond.addValueCondition(PartType::nsFullColumnName(), Query::Equals, QLatin1String("ATR"));
-        }
-
-        if (!cond.isEmpty()) {
             partQuery.addCondition(cond);
         }
 
@@ -130,16 +141,13 @@ QSqlQuery FetchHelper::buildItemQuery()
 {
     QueryBuilder itemQuery(PimItem::tableName());
 
-    itemQuery.addJoin(QueryBuilder::InnerJoin, MimeType::tableName(),
-                      PimItem::mimeTypeIdFullColumnName(), MimeType::idFullColumnName());
-
     int column = 0;
 #define ADD_COLUMN(colName, colId) { itemQuery.addColumn( colName ); mItemQueryColumnMap[colId] = column++; }
     ADD_COLUMN(PimItem::idFullColumnName(), ItemQueryPimItemIdColumn);
     if (mFetchScope.fetchRemoteId()) {
         ADD_COLUMN(PimItem::remoteIdFullColumnName(), ItemQueryPimItemRidColumn)
     }
-    ADD_COLUMN(MimeType::nameFullColumnName(), ItemQueryMimeTypeColumn)
+    ADD_COLUMN(PimItem::mimeTypeIdFullColumnName(), ItemQueryMimeTypeIdColumn)
     ADD_COLUMN(PimItem::revFullColumnName(), ItemQueryRevColumn)
     if (mFetchScope.fetchRemoteRevision()) {
         ADD_COLUMN(PimItem::remoteRevisionFullColumnName(), ItemQueryRemoteRevisionColumn)
@@ -174,8 +182,8 @@ QSqlQuery FetchHelper::buildItemQuery()
 }
 
 enum FlagQueryColumns {
-    FlagQueryIdColumn,
-    FlagQueryNameColumn
+    FlagQueryPimItemIdColumn,
+    FlagQueryFlagIdColumn
 };
 
 QSqlQuery FetchHelper::buildFlagQuery()
@@ -183,10 +191,8 @@ QSqlQuery FetchHelper::buildFlagQuery()
     QueryBuilder flagQuery(PimItem::tableName());
     flagQuery.addJoin(QueryBuilder::InnerJoin, PimItemFlagRelation::tableName(),
                       PimItem::idFullColumnName(), PimItemFlagRelation::leftFullColumnName());
-    flagQuery.addJoin(QueryBuilder::InnerJoin, Flag::tableName(),
-                      Flag::idFullColumnName(), PimItemFlagRelation::rightFullColumnName());
     flagQuery.addColumn(PimItem::idFullColumnName());
-    flagQuery.addColumn(Flag::nameFullColumnName());
+    flagQuery.addColumn(PimItemFlagRelation::rightFullColumnName());
 
     ItemQueryHelper::scopeToQuery(mScope, mConnection->context(), flagQuery);
     flagQuery.addSortColumn(PimItem::idFullColumnName(), Query::Descending);
@@ -290,16 +296,18 @@ bool FetchHelper::isScopeLocal(const Scope &scope)
     query.next();
     const QString resourceName = query.value(0).toString();
 
-    org::freedesktop::Akonadi::AgentManager manager(AkDBus::serviceName(AkDBus::Control),
-                                                    QLatin1String("/AgentManager"),
+    org::freedesktop::Akonadi::AgentManager manager(DBus::serviceName(DBus::Control),
+                                                    QStringLiteral("/AgentManager"),
                                                     DBusConnectionPool::threadConnection());
     const QString typeIdentifier = manager.agentInstanceType(resourceName);
     const QVariantMap properties = manager.agentCustomProperties(typeIdentifier);
-    return properties.value(QLatin1String("HasLocalStorage"), false).toBool();
+    return properties.value(QStringLiteral("HasLocalStorage"), false).toBool();
 }
 
 bool FetchHelper::fetchItems()
 {
+    BEGIN_TIMER(fetch)
+
     // retrieve missing parts
     // HACK: isScopeLocal() is a workaround for resources that have cache expiration
     // because when the cache expires, Baloo is not able to content of the items. So
@@ -307,7 +315,16 @@ bool FetchHelper::fetchItems()
     // cacheOnly and retrieve missing parts from the resource. However ItemRetriever
     // is painfully slow with many items and is generally designed to fetch a few
     // messages, not all of them. In the long term, we need a better way to do this.
+    BEGIN_TIMER(itemRetriever)
+    BEGIN_TIMER(scopeLocal)
+    #if ENABLE_FETCH_PROFILING
+    double scopeLocalElapsed = 0;
+    #endif
     if (!mFetchScope.cacheOnly() || isScopeLocal(mScope)) {
+        #if ENABLE_FETCH_PROFILING
+        scopeLocalElapsed = scopeLocalTimer.elapsed();
+        #endif
+
         // trigger a collection sync if configured to do so
         triggerOnDemandFetch();
 
@@ -320,19 +337,22 @@ bool FetchHelper::fetchItems()
         retriever.setChangedSince(mFetchScope.changedSince());
         if (!retriever.exec() && !mFetchScope.ignoreErrors()) {   // There we go, retrieve the missing parts from the resource.
             if (mConnection->context()->resource().isValid()) {
-                throw HandlerException(QString::fromLatin1("Unable to fetch item from backend (collection %1, resource %2) : %3")
+                throw HandlerException(QStringLiteral("Unable to fetch item from backend (collection %1, resource %2) : %3")
                                        .arg(mConnection->context()->collectionId())
                                        .arg(mConnection->context()->resource().id())
                                        .arg(QString::fromLatin1(retriever.lastError())));
             } else {
-                throw HandlerException(QString::fromLatin1("Unable to fetch item from backend (collection %1) : %2")
+                throw HandlerException(QStringLiteral("Unable to fetch item from backend (collection %1) : %2")
                                        .arg(mConnection->context()->collectionId())
                                        .arg(QString::fromLatin1(retriever.lastError())));
             }
         }
     }
+    END_TIMER(itemRetriever)
 
+    BEGIN_TIMER(items)
     QSqlQuery itemQuery = buildItemQuery();
+    END_TIMER(items)
 
     // error if query did not find any item and scope is not listing items but
     // a request for a specific item
@@ -352,35 +372,62 @@ bool FetchHelper::fetchItems()
         }
     }
     // build part query if needed
+    BEGIN_TIMER(parts)
     QSqlQuery partQuery;
     if (!mFetchScope.requestedParts().isEmpty() || mFetchScope.fullPayload() || mFetchScope.allAttributes()) {
         partQuery = buildPartQuery(mFetchScope.requestedParts(), mFetchScope.fullPayload(), mFetchScope.allAttributes());
     }
+    END_TIMER(parts)
 
     // build flag query if needed
+    BEGIN_TIMER(flags)
     QSqlQuery flagQuery;
     if (mFetchScope.fetchFlags()) {
         flagQuery = buildFlagQuery();
     }
+    END_TIMER(flags)
 
     // build tag query if needed
+    BEGIN_TIMER(tags)
     QSqlQuery tagQuery;
     if (mFetchScope.fetchTags()) {
         tagQuery = buildTagQuery();
     }
+    END_TIMER(tags)
 
+    BEGIN_TIMER(vRefs)
     QSqlQuery vRefQuery;
     if (mFetchScope.fetchVirtualReferences()) {
         vRefQuery = buildVRefQuery();
     }
+    END_TIMER(vRefs)
 
+    #if ENABLE_FETCH_PROFILING
+    int itemsCount = 0;
+    int flagsCount = 0;
+    int partsCount = 0;
+    int tagsCount = 0;
+    int vRefsCount = 0;
+    #endif
+
+    BEGIN_TIMER(processing)
+    QHash<qint64, QByteArray> flagIdNameCache;
+    QHash<qint64, QString> mimeTypeIdNameCache;
+    QHash<qint64, QByteArray> partTypeIdNameCache;
     while (itemQuery.isValid()) {
+        PROF_INC(itemsCount)
+
         const qint64 pimItemId = extractQueryResult(itemQuery, ItemQueryPimItemIdColumn).toLongLong();
         const int pimItemRev = extractQueryResult(itemQuery, ItemQueryRevColumn).toInt();
 
         Protocol::FetchItemsResponse response(pimItemId);
         response.setRevision(pimItemRev);
-        response.setMimeType(extractQueryResult(itemQuery, ItemQueryMimeTypeColumn).toString());
+        const qint64 mimeTypeId = extractQueryResult(itemQuery, ItemQueryMimeTypeIdColumn).toLongLong();
+        auto mtIter = mimeTypeIdNameCache.find(mimeTypeId);
+        if (mtIter == mimeTypeIdNameCache.end()) {
+            mtIter = mimeTypeIdNameCache.insert(mimeTypeId, MimeType::retrieveById(mimeTypeId).name());
+        }
+        response.setMimeType(mtIter.value());
         if (mFetchScope.fetchRemoteId()) {
             response.setRemoteId(extractQueryResult(itemQuery, ItemQueryPimItemRidColumn).toString());
         }
@@ -402,14 +449,19 @@ bool FetchHelper::fetchItems()
         if (mFetchScope.fetchFlags()) {
             QVector<QByteArray> flags;
             while (flagQuery.isValid()) {
-                const qint64 id = flagQuery.value(FlagQueryIdColumn).toLongLong();
+                const qint64 id = flagQuery.value(FlagQueryPimItemIdColumn).toLongLong();
                 if (id > pimItemId) {
                     flagQuery.next();
                     continue;
                 } else if (id < pimItemId) {
                     break;
                 }
-                flags << flagQuery.value(FlagQueryNameColumn).toByteArray();
+                const qint64 flagId = flagQuery.value(FlagQueryFlagIdColumn).toLongLong();
+                auto flagNameIter = flagIdNameCache.find(flagId);
+                if (flagNameIter == flagIdNameCache.end()) {
+                    flagNameIter = flagIdNameCache.insert(flagId, Flag::retrieveById(flagId).name().toUtf8());
+                }
+                flags << flagNameIter.value();
                 flagQuery.next();
             }
             response.setFlags(flags);
@@ -421,6 +473,7 @@ bool FetchHelper::fetchItems()
             //We don't take the fetch scope into account yet. It's either id only or the full tag.
             const bool fullTagsRequested = !mFetchScope.tagFetchScope().isEmpty();
             while (tagQuery.isValid()) {
+                PROF_INC(tagsCount)
                 const qint64 id = tagQuery.value(TagQueryItemIdColumn).toLongLong();
                 if (id > pimItemId) {
                     tagQuery.next();
@@ -431,12 +484,13 @@ bool FetchHelper::fetchItems()
                 tagIds << tagQuery.value(TagQueryTagIdColumn).toLongLong();
                 tagQuery.next();
             }
+
+            tags.reserve(tagIds.count());
             if (!fullTagsRequested) {
                 Q_FOREACH (qint64 tagId, tagIds) {
                     tags << Protocol::FetchTagsResponse(tagId);
                 }
             } else {
-                tags.reserve(tagIds.count());
                 Q_FOREACH (qint64 tagId, tagIds) {
                     tags << HandlerHelper::fetchTagsResponse(Tag::retrieveById(tagId));
                 }
@@ -447,6 +501,7 @@ bool FetchHelper::fetchItems()
         if (mFetchScope.fetchVirtualReferences()) {
             QVector<qint64> vRefs;
             while (vRefQuery.isValid()) {
+                PROF_INC(vRefsCount)
                 const qint64 id = vRefQuery.value(VRefQueryItemIdColumn).toLongLong();
                 if (id > pimItemId) {
                     vRefQuery.next();
@@ -472,7 +527,9 @@ bool FetchHelper::fetchItems()
                 throw HandlerException("Unable to list item relations");
             }
             QVector<Protocol::FetchRelationsResponse> relations;
-            Q_FOREACH (const Relation &rel, qb.result()) {
+            const auto result = qb.result();
+            relations.reserve(result.size());
+            Q_FOREACH (const Relation &rel, result) {
                 relations << HandlerHelper::fetchRelationsResponse(rel);
             }
             response.setRelations(relations);
@@ -487,6 +544,7 @@ bool FetchHelper::fetchItems()
         QVector<QByteArray> cachedParts;
         QVector<Protocol::StreamPayloadResponse> parts;
         while (partQuery.isValid()) {
+            PROF_INC(partsCount)
             const qint64 id = partQuery.value(PartQueryPimIdColumn).toLongLong();
             if (id > pimItemId) {
                 partQuery.next();
@@ -495,21 +553,22 @@ bool FetchHelper::fetchItems()
                 break;
             }
 
-            const QByteArray partName
-                = Utils::variantToByteArray(partQuery.value(PartQueryTypeNamespaceColumn))
-                    + ":"+ Utils::variantToByteArray(partQuery.value(PartQueryTypeNameColumn));
-
+            const qint64 partTypeId = partQuery.value(PartQueryTypeIdColumn).toLongLong();
+            auto ptIter = partTypeIdNameCache.find(partTypeId);
+            if (ptIter == partTypeIdNameCache.end()) {
+                ptIter = partTypeIdNameCache.insert(partTypeId, PartTypeHelper::fullName(PartType::retrieveById(partTypeId)).toUtf8());
+            }
             Protocol::PartMetaData metaPart;
             Protocol::StreamPayloadResponse partData;
-            partData.setPayloadName(partName);
-            metaPart.setName(partName);
+            partData.setPayloadName(ptIter.value());
+            metaPart.setName(ptIter.value());
             metaPart.setVersion(partQuery.value(PartQueryVersionColumn).toInt());
             metaPart.setSize(partQuery.value(PartQueryDataSizeColumn).toLongLong());
 
             const QByteArray data = Utils::variantToByteArray(partQuery.value(PartQueryDataColumn));
             if (mFetchScope.checkCachedPayloadPartsOnly()) {
                 if (!data.isEmpty()) {
-                    cachedParts << partName;
+                    cachedParts << ptIter.value();
                 }
                 partQuery.next();
             } else {
@@ -528,7 +587,7 @@ bool FetchHelper::fetchItems()
                 }
                 partData.setMetaData(metaPart);
 
-                if (mFetchScope.requestedParts().contains(partName) || mFetchScope.fullPayload() || mFetchScope.allAttributes()) {
+                if (mFetchScope.requestedParts().contains(ptIter.value()) || mFetchScope.fullPayload() || mFetchScope.allAttributes()) {
                     parts.append(partData);
                 }
 
@@ -550,11 +609,33 @@ bool FetchHelper::fetchItems()
 
         itemQuery.next();
     }
+    END_TIMER(processing)
 
     // update atime (only if the payload was actually requested, otherwise a simple resource sync prevents cache clearing)
+    BEGIN_TIMER(aTime)
     if (needsAccessTimeUpdate(mFetchScope.requestedParts()) || mFetchScope.fullPayload()) {
         updateItemAccessTime();
     }
+    END_TIMER(aTime)
+
+    END_TIMER(fetch)
+    #if ENABLE_FETCH_PROFILING
+    qCDebug(AKONADISERVER_LOG) << "FetchHelper execution stats:";
+    qCDebug(AKONADISERVER_LOG) << "\tItems query:" << itemsElapsed << "ms," << itemsCount << " items in total";
+    qCDebug(AKONADISERVER_LOG) << "\tFlags query:" << flagsElapsed << "ms, " << flagsCount << " flags in total";
+    qCDebug(AKONADISERVER_LOG) << "\tParts query:" << partsElapsed << "ms, " << partsCount << " parts in total";
+    qCDebug(AKONADISERVER_LOG) << "\tTags query: " << tagsElapsed << "ms, " << tagsCount << " tags in total";
+    qCDebug(AKONADISERVER_LOG) << "\tVRefs query:" << vRefsElapsed << "ms, " << vRefsCount << " vRefs in total";
+    qCDebug(AKONADISERVER_LOG) << "\t------------";
+    qCDebug(AKONADISERVER_LOG) << "\tItem retriever:" << itemRetrieverElapsed << "ms (scope local:" << scopeLocalElapsed << "ms)";
+    qCDebug(AKONADISERVER_LOG) << "\tTotal query:" << (itemsElapsed + flagsElapsed + partsElapsed + tagsElapsed + vRefsElapsed) << "ms";
+    qCDebug(AKONADISERVER_LOG) << "\tTotal processing: " << processingElapsed << "ms";
+    qCDebug(AKONADISERVER_LOG) << "\tATime update:" << aTimeElapsed << "ms";
+    qCDebug(AKONADISERVER_LOG) << "\t============";
+    qCDebug(AKONADISERVER_LOG) << "\tTotal FETCH:" << fetchElapsed << "ms";
+    qCDebug(AKONADISERVER_LOG);
+    qCDebug(AKONADISERVER_LOG);
+    #endif
 
     return true;
 }
